@@ -19,9 +19,11 @@ try:
 except:
     pass
 import torchani
+from ..utils import hartree2ev
 
-
-ev2hatree = 1/27.211385  #1 Hartree = 27.211385 eV
+torch.backends.cuda.matmul.allow_tf32 = False
+torch.backends.cudnn.allow_tf32 = False
+ev2hatree = 1/hartree2ev  
 class EnForce_ANI(torch.nn.Module):
     """Takes in an torch model, then defines forward functions for it.
     Especially useful for AIMNET (class torch.jit)
@@ -142,11 +144,11 @@ def get_mol_idx_t6(mol):
     return (idx, T)
 
 
-def calc_thermo(path: str, model_name: str, get_mol_idx_t:callable, gpu_idx=0, opt_tol=0.0005, opt_steps=5000):
-    """ASE interface for calculation thermo properties using ANI2xt and AIMNET
+def calc_thermo(path: str, model_name: str, get_mol_idx_t=None, gpu_idx=0, opt_tol=0.001, opt_steps=5000):
+    """ASE interface for calculation thermo properties using ANI2x, ANI2xt or AIMNET
     path: Input sdf file
     model_name: ANI2x, ANI2xt or AIMNET
-    get_mol_idx_t: a functioin that returns (idx, T) from a pybel mol object
+    get_mol_idx_t: a functioin that returns (idx, T) from a pybel mol object, by default using the 298 K temperature
     gpu_idx: GPU cuda index
     opt_tol: Convergence_threshold for geometry optimization
     opt_steps: Maximum geometry optimizaiton steps"""
@@ -192,7 +194,11 @@ def calc_thermo(path: str, model_name: str, get_mol_idx_t:callable, gpu_idx=0, o
         opt.run(fmax=opt_tol, steps=opt_steps)
         e = atoms.get_potential_energy()
 
-        idx, T = get_mol_idx_t(mol)
+        if get_mol_idx_t is None:
+            idx = mol.title.strip()
+            T = 298
+        else:
+            idx, T = get_mol_idx_t(mol)
 
         vib = Vibrations(atoms)
         try:
@@ -228,8 +234,8 @@ def calc_thermo(path: str, model_name: str, get_mol_idx_t:callable, gpu_idx=0, o
             vib.clean()
             mols_failed.append(mol)
 
-    print(len(mols_failed))
-    print(len(out_mols))
+    print("Number of failed thermo calculations: ", len(mols_failed))
+    print("Number of successful thermo calculations: ", len(out_mols))
     with open(outpath, 'w+') as f:
         all_mols = out_mols + mols_failed
         for mol in all_mols:
