@@ -1,8 +1,9 @@
 # Original source: /labspace/models/aimnet/batch_opt_script/
-
 import torch
+import logging
 import warnings
 import os
+import numpy as np
 try:
     import torchani
 except:
@@ -14,20 +15,22 @@ try:
 except:
     pass
 from tqdm import tqdm
+from ..utils import hartree2ev
 
-
-hartree2eV = 27.211385
+torch.backends.cuda.matmul.allow_tf32 = False
+torch.backends.cudnn.allow_tf32 = False
+# hartree2eV = 27.211385
 @torch.jit.script
 class FIRE():
     """a general optimization program """
     # For a list of documentation for different optimization programs: https://wiki.fysik.dtu.dk/ase/ase/optimize.html
     def __init__(self, coord):
         ## default parameters
-        self.dt_max = 1.0
+        self.dt_max = 0.1
         self.Nmin = 5
-        self.maxstep = 0.2
-        self.finc = 1.1
-        self.fdec = 0.5
+        self.maxstep = 0.1
+        self.finc = 1.5
+        self.fdec = 0.7
         self.astart = 0.1
         self.fa = 0.99
         self.v = torch.zeros_like(coord)
@@ -136,7 +139,7 @@ class EnForce_ANI(torch.nn.Module):
             e, f = self.ani(numbers, coord)
         elif self.name == "ANI2x":
             e = self.ani((numbers, coord)).energies
-            e = e * hartree2eV  #ANI2x (torch.models.ANI2x()) output energy unit is Hatree;
+            e = e * hartree2ev  #ANI2x (torch.models.ANI2x()) output energy unit is Hatree;
                                      #ANI ASE interface unit is eV
             g = torch.autograd.grad([e.sum()], [coord])[0]
             f = -g
@@ -167,31 +170,47 @@ class EnForce_ANI(torch.nn.Module):
             f.append(_f)
         return torch.cat(e, dim=0), torch.cat(f, dim=0)
 
-def print_stats(state):
+def print_stats(state, patience):
     """Print the optimization status"""
     numbers = state['numbers']
     num_total = numbers.size()[0]
-    num_converged = torch.sum(state['converged_mask']).to('cpu')
-    num_active = num_total - num_converged
-    print("Total 3D structures: %i  Converged: %i   Active: %i" % (num_total, num_converged, num_active))
+    num_converged_dropped = torch.sum(state['converged_mask']).to('cpu')
+    oscillating_count = state['oscilating_count'].to('cpu').reshape(-1,) >= patience
+    num_dropped = torch.sum(oscillating_count)
+    num_converged = num_converged_dropped - num_dropped
+    num_active = num_total - num_converged_dropped
+    print("Total 3D structures: %i  Converged: %i   Dropped(Oscillating): %i    Active: %i" % 
+          (num_total, num_converged, num_dropped, num_active))
+    # logging.info("Total 3D structures: %i  Converged: %i   Dropped(Oscillating): %i    Active: %i" % 
+    #       (num_total, num_converged, num_dropped, num_active))
 
-def n_steps(state, n, opttol):
+def n_steps(state, n, opttol, patience):
     """Doing n steps optimization for each input. Only converged structures are 
-    modified at each step.
-    n_steps does not change input conformer order
+    modified at each step. n_steps does not change input conformer order.
     
     Argument:
         state: an dictionary containing all information about this optimization step
-        n: optimization step"""
+        n: optimization step
+        patience: optimization stops for a conformer if the force does not decrease for a continuous patience steps"""
     # t0 = perf_counter()
     numbers = state['numbers']
     charges = state['charges']
-    num_total = numbers.size()[0]
+    # num_total = numbers.size()[0]
     coord = state['coord']
     optimizer = FIRE(coord)
+    # the following two terms are used to detect oscillating conformers
+    smallest_fmax0 = torch.tensor(np.ones((len(coord), 1)) * 999,
+                                 dtype=torch.float).to(coord.device)
+    oscilating_count0 = torch.tensor(np.zeros((len(coord), 1)),
+                                     dtype=torch.float).to(coord.device)
+    state["oscilating_count"] = oscilating_count0
+    assert(len(coord.shape) == 3)
+    assert(len(numbers.shape) == 2)
+    assert(len(charges.shape) == 1)
+    assert(len(smallest_fmax0.shape) == 2)
+    assert(len(oscilating_count0.shape) == 2)
     for istep in tqdm(range(1, (n+1), 1)):
-        not_converged = ~ state['converged_mask']  #Size fixed
-        
+        not_converged = ~ state['converged_mask']  #Essential tracker handle, size fixed
         # stop optimization if all structures converged.
         if not not_converged.any():
             break
@@ -199,14 +218,29 @@ def n_steps(state, n, opttol):
         coord = state['coord'][not_converged]  #Subset coordinates, size=not_converged.
         numbers = state['numbers'][not_converged]
         charges = state['charges'][not_converged]
+        smallest_fmax = smallest_fmax0[not_converged]
+        oscilating_count = state["oscilating_count"][not_converged]
 
         coord.requires_grad_(True)
-        e, f = state['nn'].forward_batched(coord, numbers, charges)  #Key step to calculate all coordinates and forces.
+        e, f = state['nn'].forward_batched(coord, numbers, charges)  #Key step to calculate all energies and forces.
         coord.requires_grad_(False)
 
         coord = optimizer(coord, f)
-        fmax = f.norm(dim=-1).max(dim=-1)[0]  #Size(100), Norm is the length of each vector. Here it returns the maximum force length for ecah conformer. Size (100)
-        not_converged_post = fmax > opttol
+        fmax = f.norm(dim=-1).max(dim=-1)[0]  #Tensor, Norm is the length of each vector. Here it returns the maximum force length for ecah conformer. Size (100)
+        assert(len(fmax.shape) == 1)
+        not_converged_post1 = fmax > opttol
+
+        #update smallest_fmax for each molecule
+        fmax_reduced = fmax.reshape(-1, 1) < smallest_fmax
+        fmax_reduced = fmax_reduced.reshape(-1,)
+        smallest_fmax[fmax_reduced] = fmax.reshape(-1, 1)[fmax_reduced]
+        #reduce count to 0 for reducing; raise count for non-reducing
+        oscilating_count[fmax_reduced] = 0
+        fmax_not_reduced = ~fmax_reduced
+        oscilating_count += fmax_not_reduced.reshape(-1, 1)
+        not_oscilating = oscilating_count < patience
+        not_oscilating = not_oscilating.reshape(-1,)
+        not_converged_post = not_converged_post1 & not_oscilating
 
         optimizer.clean(not_converged_post)  #Subset v, a in FIRE for next optimization
         
@@ -214,26 +248,32 @@ def n_steps(state, n, opttol):
         state['fmax'][not_converged] = fmax  #Update fmax for conformers that are optimized in this iteration
         state['energy'][not_converged] = e.detach()  #Update energy for conformers that are optimized in this iteration
         state['coord'][not_converged] = coord  #Update coordinates for conformers that are optimized in this iteration
+        smallest_fmax0[not_converged] = smallest_fmax  # update smalles_fmax for each conformer
+        state["oscilating_count"][not_converged] = oscilating_count  #update counts for continuous no reduction in fmax
     
         if (istep % (n//10)) == 0:
-            print_stats(state)
+            print_stats(state, patience)
     if istep == (n):
         print("Reaching maximum optimization step:   ", end="")
+        # logging.info("Reaching maximum optimization step:   ")
     else:
-        print(f"All structures converged at step {istep}:   ", end="")
-    print_stats(state)
+        print(f"Optimization finished at step {istep}:   ", end="")
+        # logging.info(f"Optimization finished at step {istep}:   ")
+    print_stats(state, patience)
 
 
 def ensemble_opt(net, coord, numbers, charges, param, model, device):
     """Optimizing a group of molecules
     
     Arguments:
-    net: a EnForce_ANI object
+    net: an EnForce_ANI object
     coord: coordinates of input molecules (N, m, 3). N is the number of structures
            m is the number of atoms in each structure.
-    numbers: periodict number of atoms in the molecule (include H). size is m.
+    numbers: atomic numbers in the molecule (include H). (N, m)
+    charges: (N,)
     param: a dictionary containing parameters
-    
+    model: "AIMNET", "ANI2xt" or "ANI2x"
+    device
     """
     coord = torch.tensor(coord, dtype=torch.float, device=device)
     numbers = torch.tensor(numbers, dtype=torch.long, device=device)
@@ -242,17 +282,18 @@ def ensemble_opt(net, coord, numbers, charges, param, model, device):
     fmax = torch.full(coord.shape[:1], 999.0, device=coord.device)  # size=N, a tensored filled with 999.0, representing the current maximum forces at each conformer.
     energy = torch.full(coord.shape[:1], 999.0, dtype=torch.double, device=coord.device)
     ids = torch.arange(coord.shape[0], device=coord.device)  #Returns a 1D tensor
-    optimizer = FIRE(coord)
+    # optimizer = FIRE(coord)
 
     state = dict(
         ids=ids,
         coord=coord, numbers=numbers, converged_mask=converged_mask,
-        optimizer=optimizer, nn=net, fmax=fmax, energy=energy,
+        # optimizer=optimizer, nn=net, fmax=fmax, energy=energy,
+        nn=net, fmax=fmax, energy=energy,
         timing=defaultdict(float), charges=charges,
         he=list(), close=list()  #!!! he and close?
     )
 
-    n_steps(state, param['opt_steps'], param['opttol'])
+    n_steps(state, param['opt_steps'], param['opttol'], param['patience'])
 
     return dict(
         coord=state['coord'].tolist(),
@@ -309,7 +350,6 @@ class optimizing(object):
     def __init__(self, in_f, out_f, model, device, config):
         self.in_f = in_f
         self.out_f = out_f
-        # self.energies = energies
         self.model = model
         self.device = device
         self.config = config
@@ -325,8 +365,10 @@ class optimizing(object):
         
     def run(self):
         print("Preparing for parallel optimizing... (Max optimization steps: %i)" % self.config["opt_steps"])
+        # logging.info("Preparing for parallel optimizing... (Max optimization steps: %i)" % self.config["opt_steps"])
         mols = list(pybel.readfile('sdf', self.in_f))
         print(f"Total 3D conformers: {len(mols)}")
+        # logging.info(f"Total 3D conformers: {len(mols)}")
         coord, numbers, charges = mols2lists(mols, self.model)
         if self.model == "AIMNET":
             coord_padded = padding_coords(coord, 0)
@@ -337,7 +379,6 @@ class optimizing(object):
 
         for p in self.ani.parameters():  
             p.requires_grad_(False)
-
         ani = EnForce_ANI(self.ani, self.model)  # Interesting, EnForce_ANI inherites nn.module, bu can still accept a ScriptModule object as the input
 
         with torch.jit.optimized_execution(False):
